/* vlab-cdb-create-index.c: It creates a Index Table for Chemical Database
   represented in MOL2 format.

    Copyright (C) 2001-2099, Rajkumar Buyya (rajkumar@buyya.com)!

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details @ www.gnu.org
*/

#include "header.h"

int CreateLookupFile( char *pdb_filename );
void MakeLookupFileName( char *pdbfile, char *lookupfile );

int main( int argc, char *argv[] )
{
	int i;

	printf("%s: Virtual Lab Chemical Database Indexer -- R. Buyya, 2001.\n", argv[0]);
	if( argc == 1 )
	{
		printf("Usage: %s <molecule_database_filename1> [file2 ..]\n", argv[0]);
		return 0;
	}
	for( i = 1; i < argc; i++ )
		CreateLookupFile( argv[i] );
	return 0;
}

/* Lookup Table file is same as PDB file name with extension .idx */
void MakeLookupFileName( char *pdbfile, char *lookupfile ) 
{
	int i;
	for(i = 0; pdbfile[i]; i++)
	{
		if(pdbfile[i] == '.')
			break;
		lookupfile[i] = pdbfile[i];
	}
	lookupfile[i]='\0';
	strcat(lookupfile, ".idx");
}

int CreateLookupFile( char *pdb_filename )
{
	FILE *fp;
	int idxfd; /* index file id */
	char pdb_idx_filename[BUFFSIZE];
	int i, no_more_records, use_last_header;
	char buff[MAX_LINE_LEN];
	long mol_record_no = 0;
	off_t record_location = 0; /* off_t == long long */
	int mol_record_len = 0; /* length of record */

	MakeLookupFileName(pdb_filename, pdb_idx_filename);

	if( (fp= fopen( pdb_filename, "r")) == NULL )
	{
		printf("Unable to open protein database file: %s\n", pdb_filename);;
		return 0;
	}
 	if( (idxfd = open( pdb_idx_filename, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU )) == -1 )
   {
		printf("Error: Unable to open file--%s for writting\n", pdb_idx_filename);
		return 0;
   }
	no_more_records = FALSE;
	use_last_header = FALSE;

	while(!feof(fp) && !no_more_records )
	{

	mol_record_len = 0;
	/* Skip until first "#" sign appears: start of comment */
	while(1)
	{
		/* If some records are already proceessed, then first line of next record
		   is already read, so process that first by usng last read line and then the rest */
		if( !use_last_header )
		{
			use_last_header = FALSE;
			if( fgets(buff, MAX_LINE_LEN, fp) == NULL)
			{
				no_more_records = TRUE;
				break;
			}
		}
		mol_record_len += strlen(buff);
		for( i = 0; buff[i]; i++ )
			if( buff[i] != ' ')
				break;
		if( buff[i] == '#')
			break;
	}
	if( no_more_records )
		break;
	
	/* Skip all comment lines until occurance of "@"  */
	while(1)
	{
		if( fgets(buff, MAX_LINE_LEN, fp) == NULL )
		{
			no_more_records = TRUE;
			break;
		}
		mol_record_len += strlen(buff);
	
		/* skip all prefixed spaces on a line */
		for( i = 0; buff[i]; i++ )
			if( buff[i] != ' ')
				break;

		if( buff[i] == '@')
			break;
	}
	if( no_more_records )
		break;
	/*there exist one record, so count it */
	mol_record_no += 1;

	/* read all lines until begin of next record ("#") or end of file */
	while(1)
	{
		if( fgets(buff, MAX_LINE_LEN, fp) == NULL )
		{
			no_more_records = TRUE;
			break;
		}
		for( i = 0; buff[i]; i++ )
			if( buff[i] != ' ')
				break;
		if( buff[i] == '#' )
			break;
		mol_record_len += strlen(buff);
	}
	use_last_header = TRUE;
	printf("Record #%d: Location = %ld, Length = %d\n", mol_record_no, record_location, mol_record_len);
	write(idxfd, &record_location, sizeof(off_t));
	write(idxfd, &mol_record_len, sizeof(int));
	record_location += mol_record_len;
	}
	printf("Total Number of Molecule Record found = %d\n", mol_record_no );
	fclose(fp);
	close(idxfd);
	return 0;
}
