/* worker-pool.c: mutext functions to manage maximum number of workers.
   The parameter max_workers_limit indicates maxium number of simultaneous
   requests to be served.

    Copyright (C) 2001-2099, Rajkumar Buyya (rajkumar@buyya.com)!

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details @ www.gnu.org
*/

#include "header.h"

/* global variables for use in this module only */
static pthread_mutex_t worker_mutex = PTHREAD_MUTEX_INITIALIZER;
static int active_workers = 0;   /* protected variable */
static int max_workers_limit = 0;

void err_abort(char *msg );

void setMaxWorkers( int max_workers )
{
	max_workers_limit = max_workers;
}

int anyWorkersAvailable()
{
	int ret;

	if( pthread_mutex_lock(&worker_mutex) != 0 )
		err_abort("Lock mutext");

	if( active_workers < max_workers_limit )
		ret = TRUE;
	else
		ret = FALSE;

	if( pthread_mutex_unlock(&worker_mutex) != 0 )
		err_abort("Unlock mutext");

	return ret;
}

void err_abort(char *msg )
{
	printf("Error: %s\n", msg );
	exit( 1 );
}
int AllocateWorker()
{
	int ret;

	if( pthread_mutex_lock(&worker_mutex) != 0 )
		err_abort("Lock mutext");

	if( active_workers < max_workers_limit )
	{
		active_workers++;
		ret = TRUE;
	}
	else
		ret = FALSE;

	if( pthread_mutex_unlock(&worker_mutex) != 0 )
		err_abort("Unlock mutext");
	return ret;
}
int FreeWorker()
{
	int ret;

	if( pthread_mutex_lock(&worker_mutex) != 0 )
		err_abort("Lock mutext");

	if( active_workers > 0 )
	{
		active_workers--;
		ret = TRUE;
	}
	else
		ret = FALSE;

	if( pthread_mutex_unlock(&worker_mutex) != 0 )
		err_abort("Unlock mutext");
	return ret;
}
